@@ -2,25 +2,26 @@
 from __future__ import annotations
 
 import asyncio
-from dataclasses import dataclass
 import json
 import socket
 import logging
+from dataclasses import dataclass
 
 import aiohttp
 import async_timeout
 from yarl import URL
 
-from .exceptions import (
-    EvonicConnectionClosed,
-    EvonicConnectionError,
-    EvonicConnectionTimeoutError,
-    EvonicError,
-    EvonicUnsupportedFeature,
-)
-from .models import Device
+from pyevonic.models import Device
 
-LOGGER = logging.getLogger(__package__)
+from pyevonic.exceptions import (
+    EvonicError,
+    EvonicConnectionError,
+    EvonicConnectionClosed,
+    EvonicUnsupportedFeature,
+    EvonicConnectionTimeoutError
+)
+
+LOGGER = logging.getLogger(__name__)
 
 
 @dataclass
@@ -44,33 +45,39 @@ class Evonic:
             otherwise False
         """
 
+        LOGGER.info(f"Client Connected: {self._client is not None and not self._client.closed}")
+
         return self._client is not None and not self._client.closed
 
     async def connect(self):
-        """Connect to the WebSocket of an Evonic Fire
+        """ Connect to the WebSocket of an Evonic Fire
 
         Raises:
             EvonicConnectionError: Error occurred while communicating with Evonic Fire via Websocket
         """
 
         if self.connected:
+            LOGGER.debug("Already Connected")
             return
 
         if self.session is None:
+            LOGGER.debug("No session exists, using ClientSession")
             self.session = aiohttp.ClientSession()
 
         url = URL.build(scheme="ws", host=self.host, port=81)
 
         try:
             await self.request("/modules.json", "GET", None)
-            self.available_effects()
+            await self.request("/config.options.json", "GET", None)
+            await self.__available_effects()
+            LOGGER.debug(f"Connecting Websocket to: {url}")
             self._client = await self.session.ws_connect(url=url)
             return self._client, self.session
 
         except (
-            aiohttp.WSServerHandshakeError,
-            aiohttp.ClientConnectionError,
-            socket.gaierror,
+                aiohttp.WSServerHandshakeError,
+                aiohttp.ClientConnectionError,
+                socket.gaierror,
         ) as exception:
             raise EvonicConnectionError(
                 "Error occurred while communicating with Evonic device"
@@ -78,7 +85,7 @@ class Evonic:
             ) from exception
 
     async def listen(self, callback):
-        """Listen for events on the Evonic Fire WebSocket
+        """ Listen for events on the Evonic Fire WebSocket
 
         Args:
             callback: Method to call when an update is received from the Evonic Fire
@@ -103,15 +110,16 @@ class Evonic:
                 message_data = message.json()
 
                 if self._device is None:
+                    LOGGER.debug("No Device exists, creating new instance")
                     self._device = Device(message_data)
 
                 device = self._device.update_from_dict(message_data)
                 callback(device)
 
             if message.type in (
-                aiohttp.WSMsgType.CLOSE,
-                aiohttp.WSMsgType.CLOSED,
-                aiohttp.WSMsgType.CLOSING,
+                    aiohttp.WSMsgType.CLOSE,
+                    aiohttp.WSMsgType.CLOSED,
+                    aiohttp.WSMsgType.CLOSING,
             ):
                 raise EvonicConnectionClosed(
                     f"Connection to the Evonic WebSocket on {self.host} has been closed"
@@ -120,17 +128,20 @@ class Evonic:
     async def disconnect(self):
         """Disconnect from the WebSocket of an Evonic Fire."""
         if not self._client or not self.connected:
+            LOGGER.debug("Cannot disconnect Websocket, as no client connection exists")
             return
 
         await self._client.close()
 
-    async def request(self, uri, method, data):
-        """Sends a http request to the Evonic Fire
+    async def request(self, uri, method, data, host=None, scheme=None):
+        """ Sends a http request to the Evonic Fire
 
         Args:
             uri: The URI endpoint to send request to
             method: HTTP Method
             data: Request Content
+            host:? Domain to call
+            scheme:? http vs https
 
         Raises:
             EvonicError:  Received an unexpected response from the Evonic Fire
@@ -138,14 +149,23 @@ class Evonic:
             EvonicConnectionError:  A error occurred while communicating with the Evonic Fire
         """
 
-        url = URL.build(scheme="http", host=self.host, path=uri)
+        if host is None:
+            host = self.host
+
+        if scheme is None:
+            scheme = "http"
+
+        url = URL.build(scheme=scheme, host=host, path=uri)
+
         if self.session is None:
+            LOGGER.debug("No session exists, using ClientSession")
             self.session = aiohttp.ClientSession()
             self._close_session = True
 
         try:
             async with async_timeout.timeout(self.request_timeout):
                 response = await self.session.request(method, url, json=data)
+                LOGGER.debug(f"Request Response from {url}:")
 
             content_type = response.headers.get("Content-Type", "")
             if (response.status // 100) in [4, 5]:
@@ -158,6 +178,7 @@ class Evonic:
 
             if "application/json" in content_type:
                 response_data = await response.json()
+                LOGGER.debug(response_data)
 
                 if method == "GET" and uri == "/modules.json":
                     if self._device is None:
@@ -165,22 +186,23 @@ class Evonic:
 
                     self._device.update_from_dict(data=response_data)
 
+                elif method == "GET" and uri == "/config.options.json":
+                    self._device.update_from_dict(data=response_data)
+
                 else:
-                    response_data = await response.text()
+                    response_data = await response.json()
 
         except asyncio.TimeoutError as exception:
             raise EvonicConnectionTimeoutError(
-                f"Timeout occurred while connecting to Evonic device at {self.host}"
-            ) from exception
+                f"Timeout occurred while connecting to Evonic device at {self.host}") from exception
         except (aiohttp.ClientError, socket.gaierror) as exception:
             raise EvonicConnectionError(
-                f"Error occurred while communicating with Evonic device at {self.host}"
-            ) from exception
+                f"Error occurred while communicating with Evonic device at {self.host}") from exception
 
         return response_data
 
     async def power(self, cmd):
-        """Controls the power for the Evonic Fire.
+        """ Controls the main lighting for the Evonic Fire.
 
         Args:
             cmd: The state to activate on this Fire. Can be "on", "off" or "toggle"
@@ -190,9 +212,7 @@ class Evonic:
         """
 
         if cmd not in ["on", "off", "toggle"]:
-            raise EvonicError(
-                "Command not valid. Must be one of 'on', 'off' or 'toggle'"
-            )
+            raise EvonicError("Command not valid. Must be one of 'on', 'off' or 'toggle'")
 
         if cmd == "off":
             voice_command = "Fire_OFF"
@@ -204,7 +224,7 @@ class Evonic:
         return await self.__send_voice(voice_command)
 
     async def set_effect(self, effect):
-        """Set an effect on Evonic Fire.
+        """ Set an effect on Evonic Fire.
 
         Args:
             effect: The effect to active on this Evonic Fire
@@ -213,29 +233,25 @@ class Evonic:
             EvonicUnsupportedFeature: Not a valid effect for this device
         """
         # Check effect is available for this device
-        LOGGER.info(effect)
-        LOGGER.info(self._device.effects.available_effects)
         if effect not in self._device.effects.available_effects:
             raise EvonicUnsupportedFeature("Not a valid effect for this device")
 
         return await self.__send_voice(effect)
 
     async def toggle_feature_light(self):
-        """Toggles the feature light of an Evonic Fire
+        """ Toggles the feature light of an Evonic Fire
 
         Raises:
             EvonicUnsupportedFeature: Feature Light is not supported on this device
         """
 
         if "light_box" not in self._device.info.modules:
-            raise EvonicUnsupportedFeature(
-                "Feature Light is not supported on this device"
-            )
+            raise EvonicUnsupportedFeature("Feature Light is not supported on this device")
 
         return await self.__send_voice("Light_box")
 
     async def set_light_brightness(self, rgb_id, brightness):
-        """Sets the brightness of each RGB strip
+        """ Sets the brightness of each RGB strip
 
         Args:
             rgb_id: The ID of the RGB element
@@ -254,14 +270,12 @@ class Evonic:
 
         # Must be 0 - 255
         if brightness not in range(-1, 256):
-            raise EvonicError(
-                f"{brightness} is not a valid value. Must be between 0 - 255"
-            )
+            raise EvonicError(f"{brightness} is not a valid value. Must be between 0 - 255")
 
         return await self.__send_cmd(f"rgb set {rgb_id[-1]} - - {brightness} -")
 
     async def set_animation_speed(self, rgb_id, speed):
-        """Sets the animation speed of each RGB strip
+        """ Sets the animation speed of each RGB strip
 
         Args:
             rgb_id: The ID of the RGB element
@@ -273,9 +287,7 @@ class Evonic:
         """
 
         if rgb_id not in self._device.info.modules:
-            raise EvonicUnsupportedFeature(
-                f"{rgb_id} is not a support RGB ID on this device"
-            )
+            raise EvonicUnsupportedFeature(f"{rgb_id} is not a support RGB ID on this device")
 
         if not isinstance(speed, int):
             raise EvonicError("speed must be an Integer")
@@ -287,38 +299,34 @@ class Evonic:
         return await self.__send_cmd(f"rgb set {rgb_id[-1]} - {speed} - -")
 
     async def set_temperature(self, temp):
-        """Sets the heater temperature on an Evonic Fire
+        """ Sets the heater temperature on an Evonic Fire
 
         Raises:
             EvonicUnsupportedFeature: Temperature Control is not supported on this device
         """
 
         if "temperature" not in self._device.info.modules:
-            raise EvonicUnsupportedFeature(
-                "Temperature Control is not supported on this device"
-            )
+            raise EvonicUnsupportedFeature("Temperature Control is not supported on this device")
 
         if not isinstance(temp, int):
             raise EvonicError("temp must be an Integer")
 
-        if self._device.climate.fahrenheit:
+        if self._device.info.fahrenheit:
+            LOGGER.debug("Temperature is set to Fahrenheit")
             # Must be 50 - 90
             if temp not in range(49, 91):
-                raise EvonicError(
-                    f"{temp} is not a valid value. Must be between 50 - 90"
-                )
+                raise EvonicError(f"{temp} is not a valid value. Must be between 50 - 90")
 
         else:
+            LOGGER.debug("Temperature is set to Celsius")
             # Must be 11 - 32
             if temp not in range(10, 33):
-                raise EvonicError(
-                    f"{temp} is not a valid value. Must be between 11 - 32"
-                )
+                raise EvonicError(f"{temp} is not a valid value. Must be between 11 - 32")
 
         return await self.__send_cmd(f"templevel {temp}")
 
     async def heater_power(self, cmd):
-        """Controls the Heater for the Evonic Fire.
+        """ Controls the Heater for the Evonic Fire.
 
         Args:
             cmd: The state to activate on this Fire. Can be "on", "off" or "toggle"
@@ -328,9 +336,7 @@ class Evonic:
         """
 
         if cmd not in ["on", "off", "toggle"]:
-            raise EvonicError(
-                "Command not valid. Must be one of 'on', 'off' or 'toggle'"
-            )
+            raise EvonicError("Command not valid. Must be one of 'on', 'off' or 'toggle'")
 
         if cmd == "off":
             voice_command = "Heater_OFF"
@@ -347,6 +353,7 @@ class Evonic:
         Raises:
             EvonicConnectionError:  Unable to connect to device
         """
+
         if self._device is None:
             try:
                 await self.request("/modules.json", "GET", None)
@@ -355,7 +362,7 @@ class Evonic:
         return self._device
 
     async def __send_voice(self, cmd):
-        """Sends a command via Websocket Client.
+        """ Sends a command via Websocket Client.
 
         Args:
             cmd: The value of the voice field to send
@@ -364,81 +371,51 @@ class Evonic:
         if self._client is None:
             raise Exception("Connect first")
 
+        LOGGER.debug(f"Sending voice command: {cmd}")
         return await self._client.send_str(f'{{"voice":"{cmd}"}}')
 
     async def __send_cmd(self, cmd):
-        """Sends a command to the WebSocket of an Evonic Fire
+        """ Sends a command to the WebSocket of an Evonic Fire
 
         Args:
             cmd: The cmd value to send
         """
 
+        if self._client is None:
+            raise Exception("Connect first")
+
+        LOGGER.debug(f"Sending standard command: {cmd}")
         return await self._client.send_str(f'{{"cmd":"{cmd}"}}')
 
-    def available_effects(self):
-        """Returns a list of available effects for each Evonic Fire type.
+    async def __available_effects(self):
+        """ Returns a list of available effects for each Evonic Fire type.
         Information pulled from /options.htm
         """
 
         if self._device is None:
             raise Exception("No device initialised")
 
-        configs = self._device.info.configs
-        default_effects = [
-            "Vero",
-            "Ignite",
-            "Breathe",
-            "Spectrum",
-            "Embers",
-            "Odyssey",
-            "Aurora",
-            "Red",
-            "Orange",
-            "Green",
-            "Blue",
-            "Violet",
-            "White",
-        ]
+        try:
+            LOGGER.debug("Requesting paid effects")
+            payed = await self.request(f"/effect/payed/{self._device.info.email}/{self._device.info.configs}", "GET",
+                                       None,
+                                       "evoflame.co.uk", "https")
 
-        if configs in [
-            "1800",
-            "ds1030",
-            "hal800",
-            "hal1030",
-            "hal1500",
-            "hal2400",
-            "halev4",
-            "halev8",
-            "irpanel",
-            "v630",
-            "v730",
-            "v1030",
-        ]:
+        except EvonicError as err:
+            raise EvonicConnectionError("Unable to connect to device") from err
+
+        configs = self._device.info.configs
+        default_effects = ["Vero", "Ignite", "Breathe", "Spectrum", "Embers", "Odyssey", "Aurora", "Red", "Orange",
+                           "Green", "Blue", "Violet", "White"]
+
+        if configs in ["1800", "ds1030", "hal800", "hal1030", "hal1500", "hal2400", "halev4",
+                       "halev8", "irpanel", "v630", "v730", "v1030"]:
             default_effects.insert(0, "Eos")
 
-        if configs in [
-            "ilusion2",
-            "alisio1150",
-            "alisio1550",
-            "alisio1850",
-            "alisio850",
-        ]:
-            default_effects = [
-                "Ilusion",
-                "Aurora",
-                "Patriot",
-                "Verona",
-                "Charm",
-                "Viva",
-                "Cocktail",
-                "Campfire",
-            ]
+        if configs in ["ilusion2", "alisio1150", "alisio1550", "alisio1850", "alisio850"]:
+            default_effects = ["Ilusion", "Aurora", "Patriot", "Verona", "Charm", "Viva", "Cocktail", "Campfire"]
 
-        if (
-            configs
-            in ["alente", "e1030", "e1250", "e1500", "e1800", "e2400", "e500", "e800"]
-            and configs != "1800"
-        ):
+        if configs in ["alente", "e1030", "e1250", "e1500", "e1800", "e2400", "e500", "e800"] and configs != "1800":
             default_effects = ["Evoflame", "Party"]
 
         if configs in ["sl600", "sl700", "sl1000", "sl1250", "sl1500"]:
@@ -447,11 +424,17 @@ class Evonic:
         if configs in ["video"]:
             default_effects = ["Low", "Medium", "High"]
 
-        self._device.update_from_dict({"available_effects": default_effects})
+        supported_effects = [*default_effects, *payed.get("effect")]
+        LOGGER.debug(f"Supported effects {supported_effects}")
+
+        self._device.update_from_dict({"available_effects": supported_effects})
         return
 
     async def close(self) -> None:
         """Close open client (WebSocket) session."""
+
+        LOGGER.info("Closing Websocket Session")
+
         await self.disconnect()
         if self.session and self._close_session:
             await self.session.close()
